import random,time
import os,sys
import platform as pf
curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(rootPath)
from time import sleep
import shutil

from loguru import logger
from pywebio.input import FLOAT,NUMBER,input_group,select, textarea,file_upload,checkbox,radio,actions
from pywebio.output import close_popup, output, put_file, put_html, put_image, put_markdown, put_text,popup,put_link,put_code,put_row,put_processbar,set_processbar,put_error,put_warning,toast,put_grid,put_button,put_table,use_scope,span,clear,remove
from pywebio import start_server,session,platform
from core.bladeTest.main import RemoteRunner,generateHtmlReport,running
from core.jmeterTool.swagger2jmeter import swagger2jmeter
from core.jmeterTool.har2jmeter import har2jmeter
from core.xmind2excel import makeCase
from core.utils import CFacker,getDateTime,parseJmeterXml,getTimeStamp,getDateTime,formatStr2Timestamp,timeStampStr2FormatTime,jsonPrettyOutput
from core.mqttUtil import NormalMqttGetter,NormalMqttSender
from core.kafkaUtil import general_sender,continue_orderMsg,general_orderMsg,general_orderMsgWithFilter,kafkaFetchServerWithFilter,kafkaFetchServer
from functools import partial
from multiprocessing import Process
import decimal,websockets,asyncio
import json
from functools import partial

import pywebio.output as output
import pywebio.input as inputs
import pywebio.pin as pin
from pywebio.session import hold



def kafkaSender():
    output.put_markdown("## 请输入kafka连接信息")
    pin.put_input(name='host',label='主机：端口')
    pin.put_input(name='topic',label='主题')
    pin.put_textarea(name='msg',label='消息',rows=10)
    pin.put_input(name='interval',label='发送间隔',value=0)
    def getValueAndCall():
        host=pin.pin.host
        topic=pin.pin.topic
        msg=pin.pin.msg
        interval=int(pin.pin.interval)
        
        if interval==0:
            general_sender(topic=topic,serverAndPort=host,message=msg)
            output.toast(content=f"发送成功: {host} \n {topic} \n {msg}")
        elif interval>0:
            while True:
                general_sender(topic=topic,serverAndPort=host,message=msg)
                output.toast(content=f"发送成功: {host} \n {topic} \n {msg}")
                sleep(int(interval))

    output.put_button(label='提交', onclick=lambda :getValueAndCall())


def kafkaGeter():
    put_text("json过滤如：data.deviceId")
    data = input_group("kafka连接配置", [
        inputs.input("kafka topic，必填", name="topic",value='testTopic'),
        inputs.input("kafka 地址，如ip:port，必填", name="address",value='192.168.xxx.xxx:9092'),
        inputs.input("过滤方式，仅支持填json或regx，非必填", name="filter",value='json'),
        inputs.input("过滤表达式，json使用jmeshpath方式，regx采用abc(.*)bbb的方式，非必填", name="pattern",value="payload.name"),
        inputs.input("过滤后比对关键字，过滤后的值是否等于输入的值，非必填", name="key",value="status"),
    ])
            
    for one in continue_orderMsg(data['topic'], data['address'], data['filter'], data['pattern'], data['key']):
        for a in one:
            if a is not None:
                put_text(f"{getDateTime()} : {data['topic']} --> {a}")

class DecimalEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, decimal.Decimal):
            return float(o)
        super(DecimalEncoder, self).default(o)


@use_scope('content',clear=True)
def kafkaListener():
    '''
    to send kafka message or listener the kafka topic
    :return:
    '''
    try:
        session.set_env(title='testTools')
        clear('content')
        select_type = select("选择kafka操作:",["kafka发送消息","kafka持续接收消息","kafka消息转发"])

        if select_type=="kafka发送消息":
            kafkaSender()
        elif select_type=="kafka持续接收消息":
            kafkaGeter()
        elif select_type=="kafka消息转发":
            kafkaTransfer()

    except Exception as e:
        output.popup(title="error",content=put_text(e))
        clear('content')







from functools import partial
def filterPrint(oriStr,tarStr):
    if tarStr in oriStr:
        put_text(getDateTime()+" "+oriStr)

def noFilterPrint(oriStr):
    put_text(getDateTime()+" "+oriStr)




def mqttSender():
    output.put_markdown("## 请输入mqtt连接信息")
    pin.put_input(name='host',label='mqtt主机，必填')
    pin.put_input(name='port',label='mqtt端口，必填（整数）')
    pin.put_input(name='topic',label='mqtt topic，必填')
    pin.put_input(name='user',label='mqtt 用户')
    pin.put_input(name='passwd',label='mqtt 密码')
    # pin.put_input(name='filter',label='mqtt 消息过滤字段')
    pin.put_textarea(name='msg',label='消息',rows=10)
    pin.put_input(name='interval',label='发送间隔',value=0)
    def getValueAndCall():
        host=pin.pin.host
        port=int(pin.pin.port)
        topic=pin.pin.topic
        user=pin.pin.user
        passwd=pin.pin.passwd
        msg=pin.pin.msg
        interval=int(pin.pin.interval)
        # myfilter=pin.pin.filter
        
        if interval==0:
            NormalMqttSender(host=host, port=port, topic=topic,user=user,passwd=passwd).getClient(msg)
            output.toast(content=f"发送成功: {host} \n {topic} \n {msg}")
        elif interval>0:
            while True:
                NormalMqttSender(host=host, port=port, topic=topic,user=user,passwd=passwd).getClient(msg)
                output.toast(content=f"发送成功: {host} \n {topic} \n {msg}")
                sleep(interval)
    output.put_button(label='提交', onclick=lambda :getValueAndCall())


def mqttGeter():
    data = input_group("mqtt信息",[
        inputs.input("mqtt主机，必填", name="host"),
        inputs.input("mqtt端口，必填（整数）", name="port"),
        inputs.input("mqtt topic，必填",name="topic"),
        inputs.input("mqtt用户", name="user"),
        inputs.input("mqtt密码", name="passwd"),
        inputs.input("消息包含", name="filter"),
                ])
            # put_button(label=f"{data['host']}:{data['port']}-{data['topic']}-{data['user']}:{data['passwd']}", onclick=NormalMqttGetter().close(client))
    if data['filter']=="" or data['filter']==None:
        NormalMqttGetter(host=data['host'], port=int(data['port']), topic=data['topic'],user=data['user'],passwd=data['passwd']).getClient(partial(filterPrint, tarStr=data['filter']))
    NormalMqttGetter(host=data['host'], port=int(data['port']), topic=data['topic'],user=data['user'],passwd=data['passwd']).getClient(noFilterPrint)




@use_scope('content',clear=True)
def mqttListener():
    try:
        session.set_env(title='testTools')
        clear('content')

        select_type = select("选择mqtt服务:",["自定义发送服务","自定义接收服务","本地固定服务(待定)","MQTT消息转发"])
        if select_type=="自定义接收服务":
            mqttGeter()
        elif select_type=="自定义发送服务":
            mqttSender()
        elif select_type == "本地固定服务(待定)":
            put_text('未暴露')
        elif select_type=="MQTT消息转发":
            mqttTransfer()
        
    except Exception as e:
        output.popup(title="error",content=put_text(e))
        clear('content')




def dataTransfer():
    pass
def kafkaTransfer():
    output.put_markdown("## 请输入来源kafka信息")
    pin.put_input(name='host',label='源主机：端口')
    pin.put_input(name='topic',label='源主题')
    pin.put_input(name='user',label='源用户')
    pin.put_input(name='passwd',label='源密码')
    output.put_markdown("请输入目的kafka信息")
    pin.put_input(name='host2',label='目的主机：端口')
    pin.put_input(name='topic2',label='目的主题')
    pin.put_input(name='user2',label='目的用户')
    pin.put_input(name='passwd2',label='目的密码')

    def getValueAndCall():
        host=pin.pin.host
        topic=pin.pin.topic
        user=pin.pin.user
        passwd=pin.pin.passwd

        host2=pin.pin.host2
        topic2=pin.pin.topic2
        user2=pin.pin.user2
        passwd2=pin.pin.passwd2

        for one in continue_orderMsg(topic=topic, serverAndPort=host,flag="", pattern="", key=""):
            for msg in one:
                output.toast(content=msg)
                general_sender(topic=topic2,serverAndPort=host2,message=msg)

    output.put_button(label='提交', onclick=lambda :getValueAndCall())

def mqttTransfer():
    output.put_markdown("## 请输入来源MQTT信息")
    pin.put_input(name='host',label='源主机')
    pin.put_input(name='port',label="源端口")
    pin.put_input(name='topic',label='源主题')
    pin.put_input(name='user',label='源用户')
    pin.put_input(name='passwd',label='源密码')
    output.put_markdown("## 请输入目的MQTT信息")
    pin.put_input(name='host2',label='目的主机')
    pin.put_input(name='port2',label="目的端口")
    pin.put_input(name='topic2',label='目的主题')
    pin.put_input(name='user2',label='目的用户')
    pin.put_input(name='passwd2',label='目的密码')

    def getValueAndCall():
        host=pin.pin.host
        port=pin.pin.port
        topic=pin.pin.topic
        user=pin.pin.user
        passwd=pin.pin.passwd

        host2=pin.pin.host2
        port2=pin.pin.port2
        topic2=pin.pin.topic2
        user2=pin.pin.user2
        passwd2=pin.pin.passwd2

        NormalMqttGetter(host=host, port=port, topic=topic,user=user,passwd=passwd).getClient(partial(filterPrint, tarStr=data['filter']))
    output.put_button(label='提交', onclick=lambda :getValueAndCall())



def timeStampGetter():
    output.set_scope('time' ,if_exist='remove')
    output.put_markdown("# 当前时间：",scope='time')
    output.put_text("- "+str(getDateTime()),scope='time')
    def refreshTime():
        output.put_text("- "+str(getDateTime()),scope='time')
    output.put_button(label='刷新时间', onclick=lambda :refreshTime())

    output.set_scope('time2',if_exist='remove')
    output.put_markdown("# 当前时间戳：",scope='time2')
    output.put_text("- "+str(getTimeStamp()),scope='time2')
    def refreshTimestamp():
        output.put_text("- "+str(getTimeStamp()),scope='time2')
    output.put_button(label='刷新时间戳', onclick=lambda :refreshTimestamp())

    output.set_scope('time3',if_exist='remove')
    output.put_markdown("## 时间转时间戳：",scope='time3')
    pin.put_input(name='curTime',label='请输入格式化时间',value="请输入格式化时间：2022-01-01 23:59:59.999",scope='time3')
    def time2stamp():
        curTime=pin.pin.curTime
        output.put_text(formatStr2Timestamp(curTime),scope='time3')
    output.put_button(label='确定', onclick=lambda :time2stamp())

    output.set_scope('time4',if_exist='remove')
    output.put_markdown("## 时间戳转时间：",scope='time4')
    pin.put_input(name='timeStamp',label='请输入时间戳',value="请输入13位时间戳，不够时后三位可写位0",scope='time4')
    def stamp2time():
        timeStamp=pin.pin.timeStamp
        output.put_text(timeStampStr2FormatTime(timeStamp),scope='time4')
    output.put_button(label='确定', onclick=lambda :stamp2time())
    


def jsonFormater():
    output.put_markdown("## Json格式化：")
    pin.put_textarea(name='oriJson',rows=10,help_text='请输入Json')
    def getValue():
        oriStr=pin.pin.oriJson
        output.put_text(jsonPrettyOutput(oriStr))
    output.put_button(label='格式化', onclick=lambda :getValue())




if __name__ == '__main__':
    start_server(mqttListener,port=8999,debug=True,cdn=False,auto_open_webbrowser=True)





#!/anaconda3/envs/xxx/bin python3.7
# -*- coding: utf-8 -*-
# ---
# @Software: PyCharm
# @File: main.py
# @Author: oupeng
# @Time: 9月 23, 2021
# ---
import json
import time
import os
from fabric import Connection,SerialGroup
from loguru import logger
from requests import session
from copy import deepcopy
import csv
import pytest
import traceback

from jinja2 import Environment, FileSystemLoader


# class setEnvs():
#     f=open('.env','r')
#     for one in f.readlines():
#         k,v=one.split('=')
#         os.environ[k]=v


class RunnerResult(object):
    '''
    this just for generate result for the remote executor
    '''
    def __init__(self):
        
        self.testResTemp={"testTitle":"","chaosCommand":"","chaosId":"","checkCommand":"","expectResponse":"","realResponse":"","checkStatus":""}
        self.testResList=[]
    
    def clearTemp(self):
        self.testResTemp={"testTitle":"","chaosCommand":"","chaosId":"","checkCommand":"","expectResponse":"","realResponse":"","checkStatus":""}
    
    def clearResList(self):
        self.testResList=[]

    def printResList(self):
        for one in self.testResList:
            print('####################')
            for k,v in one.items():
                print(f"{k}-->{v}")
    
    def printResultTemp(self):
        for k,v in self.testResTemp.items():
                print(f"{k}-->{v}")

    def generaterReport(self):
        pass


class RemoteRunner(object):
    '''
    run command in remote host
    '''
    def __init__(self, host, port, user, passwd):
        '''

        :param host:
        :param port:
        :param user:
        :param passwd:
        result will be store in instance res variables
        '''
        self.host = host
        self.port = port
        self.user = user
        self.passwd = {"password": passwd}
        self.conn = Connection(host=self.host, user=self.user, port=self.port, connect_kwargs=self.passwd)
        self.res=""
        self.uid=None
        self.containerList=[]
        self.commandList=None
        self.testResult=RunnerResult()
        


    def __enter__(self):
        return self
        

    def __exit__(self,exc_type,exc_val,exc_tb):
        self.conn.close()

    def upload_file(self, localFilePath, remoteFilePath):
        '''
        :param localFilePath:
        :param remoteFilePath:
        :return:
        '''
        # logger.info(f"localFilePath: {localFilePath}")
        # logger.info(f"remoteFilePath: {remoteFilePath}")
        logger.info(f"localFilePath: {localFilePath}, remoteFilePath: {remoteFilePath}")
        self.conn.put(local=localFilePath, remote=remoteFilePath)
        return self

    def download_file(self, localFilePath, remoteFilePath):
        '''

        :param localFilePath:
        :param remoteFilePath:
        :return:
        '''
        logger.info(f"localFilePath: {localFilePath}, remoteFilePath: {remoteFilePath}")
        # logger.info(f"remoteFilePath: {remoteFilePath}")
        self.conn.get(remote=remoteFilePath, local=localFilePath)
        return self


    def get_chaos_id(self):
        if "success" in self.res.stdout.strip():
            id=json.loads(self.res.stdout.strip())['result']
            print(f"chaos id: < {id} >")
            self.uid=id
            
        return self

    def exec_cmd(self, cmd):
        '''
        :param cmd: command in running on the host
        :return:
        '''
        try:
            logger.info(f"exec cmd: < {cmd} >")
            self.commandList=cmd
            res = self.conn.run(cmd)
            # logger.info(f'cmd stdout is: {res.stdout.strip()}')
            # logger.error(f'cmd stdout is: {res.stderr.strip()}')
            self.res=res
            self.get_chaos_id()
            return self
        except Exception as e:
            logger.warning(e)

    def exec_on_multi(self,*hosts,cmd):
        '''

        :param hosts: host list,contain lots of host
        :param cmd: command in running on the host
        :return:
        '''
        logger.info(f"hosts: <{hosts}>")
        logger.info(f"multi cmd: <{cmd}>")
        self.commandList=cmd
        res=SerialGroup(hosts).run(cmd)
        # logger.info(f'cmd stdout is: {res.stdout.strip()}')
        # logger.error(f'cmd stdout is: {res.stderr.strip()}')
        self.res=res
        self.get_chaos_id()
        return self

    def exec_check_by_cmd(self,cmd,expect):
        '''

        :param cmd: command
        :param expect: command expect str
        :return:
        '''
        logger.info(f"check cmd is: < {cmd} >")
        self.exec_cmd(cmd)
        stdout = self.res.stdout.strip()
        logger.info(f"checking: result is: < {stdout} >, expect is: < {expect} >")
        # assert expect in stdout
        if expect in stdout or expect < stdout:
            return True
        else:
            
            return False

    def exec_check_contain(self,expect):
        '''
        :param expect: expect str
        :return:
        '''
        stdout=self.res.stdout.strip()
        logger.info(f"checking: result is: < {stdout} >, expect is: < {expect} >")
        # assert expect in stdout
        if expect in stdout:
            # logger.info(f"expect: {expect}")
            # logger.info(f"stdout: {stdout}")
            
            return True
        
        return False

    def exec_check_equal(self,expect):
        '''

        :param expect: expect str, exactly same
        :return:
        '''
        stdout=self.res.stdout.strip()
        logger.info(f"checking: result is: < {stdout} >, expect is: < {expect} >")
        # assert expect == stdout
        if expect == stdout:
            # logger.info(f"expect: {expect}")
            # logger.info(f"stdout: {stdout}")
            
            return True
        
        return False



    def exec_check_request_response(self,httpObj,expect="",*kwargs):
        '''
        check service ok, using response code by contain
        :param url:
        :param method:
        :param data:
        :param json:
        :param expect:
        :param kwargs:
        :return:
        '''
        '''
             url_str=http://mobile.abc.org/sfsf~post~urlparams~jsonparms~headers
             http://http://192.168.xxx.xxx:21368/api-dev/terminus-security/account/login~post~ ~{"loginName":"$loginName","password":"$loginPass","businessCode":"terminus","loginType":"ACCOUNT","ticket":"7c9db59cdc189b5d56ba21bf20e355fcc30c0049","rightCode":"dfdf"}~{"Content-Type": "application/json;charset=UTF-8"}
            '''
        # logger.info(type(httpObj))
        # logger.info(httpObj)
        logger.info(f'url is {httpObj["url"]}')
        logger.info(f'method is {httpObj["method"]}')
        logger.info(f'params is {httpObj["params"]}')
        logger.info(f'jsonstr is {httpObj["json"]}')
        logger.info(f'headers is {httpObj["header"]}')
        # requestFuncStr=f"session().{method}(url='{url}',data={params},json={jsonstr})"
        start_time=time.time()
        resp = session().request(url=httpObj['url'], method=httpObj['method'], params=httpObj['params'], data=httpObj['json'], headers=httpObj['header'])
        logger.info(f"http response: < {resp.text} >")
        logger.info(f"expect string is: < {expect} >")
        end_time=time.time()
        time_spend=end_time-start_time
        logger.info(f"request spend: < {time_spend} > ")


        # print(requestFuncStr)
        if resp.status_code != 200:
            
            return False
        else:
            if expect in resp.text:
                
                return True
                # assert True
            else: 
                
                return False
                # assert False

    def prepare_blade(self):
        '''
        :return: blade binaries will be placed in /opt/chaos/chaosblade-1.2.0/, and add blade in path
        '''
        self.exec_cmd("cd /opt;rm -rf chaos*")
        self.upload_file("chaosblade.tar.gz", "/opt")
        self.exec_cmd('cd /opt; mkdir chaos;tar -xzvf chaosblade.tar.gz -C ./chaos;')
        self.exec_cmd('echo "export PATH=$PATH:/opt/chaos/chaosblade-1.2.0/">>~/.bashrc')
        self.exec_cmd('source ~/.bashrc')
        return self

    def check_blade_exist(self):
        if self.exec_cmd("echo $PATH|grep chaosblade"):
            return True
        else:
            return False


    def get_docker_id(self):
        '''
        get docker id
        :return:
        '''
        res=self.conn.run("docker ps|awk '{if (NR>2) {print $1}}'")
        self.containerList=deepcopy(res.stdout.strip().split('\n'))
        logger.info(f"container list is: < {self.containerList} >")
        return self
        # print(type(self.containerList))
        # return self.containerList

    def run_docker_cmd_by_dockerName(self,cmd,uniq_docker_name):
        '''
        run docker command by name
        :param cmd:
        :param uniq_docker_name:
        :return:
        '''
        docker_id=self.conn.run("docker ps|grep "+uniq_docker_name+"|awk '{print $1}'").stdout.strip()
        logger.info(f"docker_id is:< {docker_id}>")
        logger.info(f"docker command :< {cmd} --container-id {docker_id} > will be execute")
        self.commandList=f"{cmd} --container-id {docker_id}"
        self.res=self.conn.run(f"{cmd} --container-id {docker_id}")
        self.get_chaos_id()
        return self

    def run_docker_cmd_by_dockerId(self,cmd,docker_id):
        '''
        run docker command by id
        :param cmd:
        :param docker_id:
        :return:
        '''
        # docker_id=self.conn.run("docker ps|grep "+uniq_docker_name+"|awk '{print $1}'").stdout.strip()
        logger.info(f"docker command: <{cmd} --container-id {docker_id} > will be execute")
        self.commandList=f"{cmd} --container-id {docker_id}"
        self.res=self.conn.run(f"{cmd} --container-id {docker_id}")
        self.get_chaos_id()
        return self

    def run_docker_cmd(self,cmd):
        '''
        run docker commmand directly by writed case
        :param cmd:
        :return:
        '''
        # docker_id=self.conn.run("docker ps|grep "+uniq_docker_name+"|awk '{print $1}'").stdout.strip()
        logger.info(f"docker command: < {cmd} > will be execute")
        self.commandList=f"{cmd}"
        self.res=self.conn.run(f"{cmd}")
        self.get_chaos_id()
        return self

    def destroy_chaos(self):
        logger.info(f"id:< {self.uid} > will destroy")
        self.conn.run(f"blade destroy {self.uid}")




def running(wait=1,jf=None):
    
    resList=[]
    # logger.info(f'\n' * 5)
    # logger.info("\n开始"+f'--' * 50)
    # f=open("config.json",encoding='utf-8')
    # jf=json.load(f)
    for one in jf['data']:
        # logger.info(f'\n\n\n\n@@@读取到的任务：@@@\n:{one["description"]}')
        try:
            if one['flag'] == "true":
                
                logger.info(f'\n+++++++将执行的任务：+++++++ :\n{one["description"]}')
                host=one['host']
                port=int(one['port'])
                user=one['user']
                passwd=one['passwd']

                r = RemoteRunner(host, port, user, passwd)
                if not r.check_blade_exist():
                    r.prepare_blade()

                r.testResult.testResTemp['testTitle']=one["description"]
                
                if one['httpCheckCommand'] == "":
                    if one['dockerName'] != "":
                        # r = RemoteRunner(host=host, port=port, user=user, passwd=passwd)
                        r.run_docker_cmd_by_dockerName(one['execCommand'], one['dockerName']).exec_check_contain(
                            one['execCommandExpect'])
                        r.testResult.testResTemp['realResponse']=r.res
                        r.testResult.testResTemp['chaosCommand']=r.commandList
                        r.testResult.testResTemp['chaosId']=r.uid
                        time.sleep(wait)
                        r.testResult.testResTemp["checkStatus"]=r.exec_check_by_cmd(one['checkCommand'], one['checkExpect'])
                        r.testResult.testResTemp['checkCommand']=one['checkCommand']
                        r.testResult.testResTemp['expectResponse']=one['checkExpect']
                        time.sleep(wait)
                        r.destroy_chaos()
                    else:
                        if "docker" in one['execCommand']:
                            # r = RemoteRunner(host=host, port=port, user=user, passwd=passwd)
                            r.run_docker_cmd(one['execCommand']).exec_check_contain(one['execCommandExpect'])
                            r.testResult.testResTemp['realResponse']=r.res
                            time.sleep(wait)
                            r.testResult.testResTemp['chaosCommand']=r.commandList
                            r.testResult.testResTemp['chaosId']=r.uid
                            r.testResult.testResTemp["checkStatus"]=r.exec_check_by_cmd(one['checkCommand'], one['checkExpect'])
                            r.testResult.testResTemp['realResponse']=r.res
                            r.testResult.testResTemp['checkCommand']=one['checkCommand']
                            r.testResult.testResTemp['expectResponse']=one['checkExpect']
                            r.testResult.testResTemp['realResponse']=r.res
                            time.sleep(wait)
                            r.destroy_chaos()
                        else:
                            # r = RemoteRunner(host=host, port=port, user=user,passwd=passwd)
                            r.exec_cmd(one['execCommand']).exec_check_contain(one['execCommandExpect'])
                            # r.testResult.testResTemp['realResponse']=r.res
                            time.sleep(wait)
                            r.testResult.testResTemp['chaosCommand']=r.commandList
                            r.testResult.testResTemp['chaosId']=r.uid
                            r.testResult.testResTemp["checkStatus"]=r.exec_check_by_cmd(one['checkCommand'], one['checkExpect'])
                            r.testResult.testResTemp['realResponse']=r.res
                            r.testResult.testResTemp['checkCommand']=one['checkCommand']
                            r.testResult.testResTemp['expectResponse']=one['checkExpect']
                            time.sleep(wait)
                            r.destroy_chaos()
                else:
                    
                    if one['dockerName'] != "":
                        # r = RemoteRunner(host=host, port=port, user=user, passwd=passwd)
                        r.testResult.testResTemp["checkStatus"]=r.run_docker_cmd_by_dockerName(one['execCommand'], one['dockerName']).exec_check_request_response(one['httpCheckCommand'],one['httpExpect'])
                        r.testResult.testResTemp['realResponse']=r.res
                        r.testResult.testResTemp['chaosCommand']=r.commandList
                        r.testResult.testResTemp['chaosId']=r.uid
                        r.testResult.testResTemp['checkCommand']=one['httpCheckCommand']
                        r.testResult.testResTemp['expectResponse']=one['httpExpect']
                        time.sleep(wait)
                        # r.exec_check_by_cmd(one['checkCommand'], one['checkExpect'])
                        time.sleep(wait)
                        r.destroy_chaos()
                    else:
                        if "docker" in one['execCommand']:
                            # r = RemoteRunner(host=host, port=port, user=user,passwd=passwd)
                            r.testResult.testResTemp["checkStatus"]=r.run_docker_cmd(one['execCommand']).exec_check_request_response(one['httpCheckCommand'],one['httpExpect'])
                            r.testResult.testResTemp['realResponse']=r.res
                            r.testResult.testResTemp['chaosCommand']=r.commandList
                            r.testResult.testResTemp['chaosId']=r.uid
                            r.testResult.testResTemp['checkCommand']=one['httpCheckCommand']
                            r.testResult.testResTemp['expectResponse']=one['httpExpect']
                            time.sleep(wait)
                            # r.exec_check_by_cmd(one['checkCommand'], one['checkExpect'])
                            time.sleep(wait)
                            r.destroy_chaos()
                        else:
                            # r = RemoteRunner(host=one['host'], port=port, user=user,passwd=passwd)
                            r.testResult.testResTemp["checkStatus"]=r.exec_cmd(one['execCommand']).exec_check_request_response(one['httpCheckCommand'],one['httpExpect'])
                            r.testResult.testResTemp['realResponse']=r.res
                            r.testResult.testResTemp['chaosCommand']=r.commandList
                            r.testResult.testResTemp['chaosId']=r.uid
                            r.testResult.testResTemp['checkCommand']=one['httpCheckCommand']
                            r.testResult.testResTemp['expectResponse']=one['httpExpect']
                            time.sleep(wait)
                            # r.exec_check_by_cmd(one['checkCommand'], one['checkExpect'])
                            time.sleep(wait)
                            r.destroy_chaos()
                    # requestStringParser(url_str=one['httpCheck'],loginRequire="no",checkResponse=one['httpExpect'])
                resList.append(r.testResult.testResTemp)
        except Exception as e:
            r.testResult.testResTemp['testTitle']=one["description"]
            r.testResult.testResTemp['chaosCommand']=one["execCommand"]
            r.testResult.testResTemp["realResponse"]=traceback.format_exc()
            r.testResult.testResTemp["checkStatus"]=False
            resList.append(r.testResult.testResTemp)
                # r.testResult.printResultTemp()
                # r.testResult.clearTemp()
        r.testResult.printResultTemp()
        r.testResult.clearTemp()
    logger.info("\n全部完成："+f'--' * 50)
    return resList

def generateHtmlReport(resList):
    from datetime import datetime
    passCounter=0
    failCounter=0
    total=0
    for one in resList:
        if one["checkStatus"]==True:
            passCounter+=1
        else:
            failCounter+=1
        total+=1

    exeTime=datetime.strftime(datetime.now(),"%y-%m-%d %H:%M:%S")

    env = Environment(loader=FileSystemLoader('./bootstrap'))
    template = env.get_template('template.html')
    with open("testResult.html", 'w+', encoding='utf-8') as f:
        out = template.render(resList=resList,exeTime=exeTime,passCounter=passCounter,failCounter=failCounter,total=total)
        f.write(out)
    return os.path.abspath('.')+os.path.sep+'testResult.html'



if __name__ == "__main__":
    generateHtmlReport(running(jf=json.load(open("config.json",encoding='utf-8'))))




import random,time
import os,sys
import platform as pf
curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(rootPath)
from time import sleep
import shutil

from loguru import logger
from pywebio.input import input, FLOAT,NUMBER,input_group,select, textarea,file_upload,checkbox,radio,actions
from pywebio.output import popup, close_popup, output, put_file, put_html, put_image, put_markdown, put_text,popup,put_link,put_code,put_row,put_processbar,set_processbar,put_error,put_warning,toast,put_grid,put_button,put_table,use_scope,span,clear,remove
from pywebio import start_server,session,platform
from core.bladeTest.main import RemoteRunner,generateHtmlReport,running
from core.jmeterTool.swagger2jmeter import swagger2jmeter
from core.jmeterTool.har2jmeter import har2jmeter
from core.xmind2excel import makeCase
from core.utils import CFacker,getDateTime,parseJmeterXml
from core.mqttUtil import NormalMqttGetter,NormalMqttSender
from core.kafkaUtil import general_sender,continue_orderMsg,general_orderMsg,general_orderMsgWithFilter,kafkaFetchServerWithFilter,kafkaFetchServer
from functools import partial
from multiprocessing import Process
import decimal,websockets,asyncio
import json
from functools import partial

@use_scope('content',clear=True)
def onePageInput():
    clear('content')
    '''
    using formated json script to generate the blade test and run it
    :return:
    '''
    try:
        session.set_env(title='testToolKit')
        clear('content')
        put_markdown(r'''# 基础指令参考：
            ## blade create cpu load [flags]
                                --timeout string   设定运行时长，单位是秒，通用参数
                                --cpu-count string     指定 CPU 满载的个数
                                --cpu-list string      指定 CPU 满载的具体核，核索引从 0 开始 (0-3 or 1,3)
                                --cpu-percent string   指定 CPU 负载百分比，取值在 0-100
            ##                blade create disk burn
                                --path string      指定提升磁盘 io 的目录，会作用于其所在的磁盘上，默认值是 /
                                --read             触发提升磁盘读 IO 负载，会创建 600M 的文件用于读，销毁实验会自动删除
                                --size string      块大小, 单位是 M, 默认值是 10，一般不需要修改，除非想更大的提高 io 负载
                                --timeout string   设定运行时长，单位是秒，通用参数
                                --write            触发提升磁盘写 IO 负载，会根据块大小的值来写入一个文件，比如块大小是 10，则固定的块的数量是 100，则会创建 1000M 的文件，销毁实验会自动删除
            ##                blade create disk fill
                                --path string      需要填充的目录，默认值是 /
                                --size string      需要填充的文件大小，单位是 M，取值是整数，例如 --size 1024
                                --reserve string   保留磁盘大小，单位是MB。取值是不包含单位的正整数，例如 --reserve 1024。如果 size、percent、reserve 参数都存在，优先级是 percent > reserve > size
                                --percent string   指定磁盘使用率，取值是不带%号的正整数，例如 --percent 80
                                --retain-handle    是否保留填充
                                --timeout string   设定运行时长，单位是秒，通用参数
            ##                blade create mem load
                                --mem-percent string    内存使用率，取值是 0 到 100 的整数
                                --mode string   内存占用模式，有 ram 和 cache 两种，例如 --mode ram。ram 采用代码实现，可控制占用速率，优先推荐此模式；cache 是通过挂载tmpfs实现；默认值是 --mode cache
                                --reserve string    保留内存的大小，单位是MB，如果 mem-percent 参数存在，则优先使用 mem-percent 参数
                                --rate string 内存占用速率，单位是 MB/S，仅在 --mode ram 时生效
                                --timeout string   设定运行时长，单位是秒，通用参数
            ##                blade create network delay
                                --destination-ip string   目标 IP. 支持通过子网掩码来指定一个网段的IP地址, 例如 192.168.1.0/24. 则 192.168.1.0~192.168.1.255 都生效。你也可以指定固定的 IP，如 192.168.1.1 或者 192.168.1.1/32，也可以通过都号分隔多个参数，例如 192.168.1.1,192.168.2.1。
                                --exclude-port string     排除掉的端口，默认会忽略掉通信的对端端口，目的是保留通信可用。可以指定多个，使用逗号分隔或者连接符表示范围，例如 22,8000 或者 8000-8010。 这个参数不能与 --local-port 或者 --remote-port 参数一起使用
                                --exclude-ip string       排除受影响的 IP，支持通过子网掩码来指定一个网段的IP地址, 例如 192.168.1.0/24. 则 192.168.1.0~192.168.1.255 都生效。你也可以指定固定的 IP，如 192.168.1.1 或者 192.168.1.1/32，也可以通过都号分隔多个参数，例如 192.168.1.1,192.168.2.1。
                                --interface string        网卡设备，例如 eth0 (必要参数)
                                --local-port string       本地端口，一般是本机暴露服务的端口。可以指定多个，使用逗号分隔或者连接符表示范围，例如 80,8000-8080
                                --offset string           延迟时间上下浮动的值, 单位是毫秒
                                --remote-port string      远程端口，一般是要访问的外部暴露服务的端口。可以指定多个，使用逗号分隔或者连接符表示范围，例如 80,8000-8080
                                --time string             延迟时间，单位是毫秒 (必要参数)
                                --force                   强制覆盖已有的 tc 规则，请务必在明确之前的规则可覆盖的情况下使用
                                --ignore-peer-port        针对添加 --exclude-port 参数，报 ss 命令找不到的情况下使用，忽略排除端口
                                --timeout string          设定运行时长，单位是秒，通用参数
            ##                blade create network loss
                                --destination-ip string   目标 IP. 支持通过子网掩码来指定一个网段的IP地址, 例如 192.168.1.0/24. 则 192.168.1.0~192.168.1.255 都生效。你也可以指定固定的 IP，如 192.168.1.1 或者 192.168.1.1/32，也可以通过都号分隔多个参数，例如 192.168.1.1,192.168.2.1。
                                --exclude-port string     排除掉的端口，默认会忽略掉通信的对端端口，目的是保留通信可用。可以指定多个，使用逗号分隔或者连接符表示范围，例如 22,8000 或者 8000-8010。 这个参数不能与 --local-port 或者 --remote-port 参数一起使用
                                --exclude-ip string       排除受影响的 IP，支持通过子网掩码来指定一个网段的IP地址, 例如 192.168.1.0/24. 则 192.168.1.0~192.168.1.255 都生效。你也可以指定固定的 IP，如 192.168.1.1 或者 192.168.1.1/32，也可以通过都号分隔多个参数，例如 192.168.1.1,192.168.2.1。
                                --interface string        网卡设备，例如 eth0 (必要参数)
                                --local-port string       本地端口，一般是本机暴露服务的端口。可以指定多个，使用逗号分隔或者连接符表示范围，例如 80,8000-8080
                                --percent string          丢包百分比，取值在[0, 100]的正整数 (必要参数)
                                --remote-port string      远程端口，一般是要访问的外部暴露服务的端口。可以指定多个，使用逗号分隔或者连接符表示范围，例如 80,8000-8080
                                --force                   强制覆盖已有的 tc 规则，请务必在明确之前的规则可覆盖的情况下使用
                                --ignore-peer-port        针对添加 --exclude-port 参数，报 ss 命令找不到的情况下使用，忽略排除端口
                                --timeout string          设定运行时长，单位是秒，通用参数
            ##                blade create network occupy
                                --port string             指定被占用的端口，（必填项）
                                --force                   强制占用此端口，会将已使用此端口的进程杀掉
                                --timeout string          设定运行时长，单位是秒，通用参数
            ##                blade create process kill
                                --process string       进程关键词，会在整个命令行中查找
                                --process-cmd string   进程命令，只会在命令中查找
                                --count string      限制杀掉进程的数量，0 表示无限制
                                --signal string     指定杀进程的信号量，默认是 9，例如 --signal 15
                                --timeout string   设定运行时长，单位是秒，通用参数
            ##                blade create process stop
                                --process string       进程关键词，会在整个命令行中查找
                                --process-cmd string   进程命令，只会在命令中查找
                                --timeout string   设定运行时长，单位是秒，通用参数
            ##                blade create docker cpu
                                --blade-override           是否覆盖容器内已有的 chaosblade 工具，默认是 false，表示不覆盖，chaosblade 在容器内的部署路径为 /opt/chaosblade
                                --blade-tar-file string    指定本地 chaosblade-VERSION.tar.gz 工具包全路径，用于拷贝到容器内执行
                                --container-id string      目标容器 ID
                                --docker-endpoint string   Docker server 地址，默认为本地的 /var/run/docker.sock
            ##                blade create docker network
                                --container-id string      目标容器 ID
                                --docker-endpoint string   Docker server 地址，默认为本地的 /var/run/docker.sock
                                --image-repo string        chaosblade-tool 镜像仓库地址，默认是从 `registry.cn-hangzhou.aliyuncs.com/chaosblade`
            ##                blade create docker process
                                --blade-override           是否覆盖容器内已有的 chaosblade 工具，默认是 false，表示不覆盖，chaosblade 在容器内的部署路径为 /opt/chaosblade
                                --blade-tar-file string    指定本地 chaosblade-VERSION.tar.gz 工具包全路径，用于拷贝到容器内执行
                                --container-id string      目标容器 ID
                                --docker-endpoint string   Docker server 地址，默认为本地的 /var/run/docker.sock
            ##                blade create docker container
                                --container-id string      要删除的容器 ID
                                --docker-endpoint string   Docker server 地址，默认为本地的 /var/run/docker.sock
                                --force                    是否强制删除
            ''', lstrip=True).show()
        json_str = textarea('请输入json串',rows=30,value=r'''{
        "data":[
        {
            "description": "主机网络限制延迟查看接口响应",
            "flag":"true", 是否运行
            "type": "docker", 类型是host 或者 docker
            "host":"192.168.xxxx.xxx",
            "port": 22,
            "user":"root",
            "passwd": "xxxxxx",
            "execCommand":"blade create network delay --time 1000 --interface eth0 --local-port 18086", 需要执行的命令
            "execCommandExpect": "success", 通常为success，可以不用改
            "dockerName": "",  如果是docker需要知道docker名称，一定要是唯一标识
            "checkCommand": "", 执行命令后的检查命令，比如CPU是否达到 70%
            "checkExpect": "", 检查的期望值
            "httpCheckCommand": { 通过http的方式进行检查，以及发送的内容，目前不支持需要登录的方式，建议手动添加token到header
            "url":"http://192.168.xxxx.xxx:18086/query",
            "method":"POST",
            "header":{},
            "cookie":{},
            "params":{"q":"show databases"},
            "json":{}
            },
            "httpExpect": "results" http检查值的返回
        },{}]}''',code={
                'mode': "json",
                'theme': 'darcula'
            })
        put_code(json_str, language='json') 
        runAndGetReport(json.loads(json_str))
    except Exception as e:
        popup(title="error",content=put_text(e))
        clear('content')







@use_scope('content',clear=True)
def oneCheck():
    clear('content')
    '''
    using step by step way to execute blade test
    :return:
    '''
    try:
        session.set_env(title='testToolKit')
        #clear('content')
        input_data={"data":[]}
        temp_data={}
        
        go_on=True
        while go_on:
            http_check_data={}
            type_data={}

            server_data = input_group("请输入主机信息",[
            input('本次测试描述', name="description"),
            input('输入服务器IP', name="host"),
            input('输入服务器端口', name="port", type=NUMBER),
            input('输入服务器用户名', name="user"),
            input('输入服务器密码', name="passwd"),
            ])

            r=RemoteRunner(server_data["host"],server_data["port"],server_data["user"],server_data["passwd"])
            upload = select("是否已经上传chaos文件:",["是","否"])
            if upload=="否":
                popup(title="注意",content="部署命令安装中。。。")
                r.exec_cmd("cd /opt;rm -rf chaos*")
                r.upload_file("chaosblade.tar.gz", "/opt")
                r.exec_cmd('cd /opt; mkdir chaos;tar -xzvf chaosblade.tar.gz -C ./chaos;')
                r.exec_cmd('echo "export PATH=$PATH:/opt/chaos/chaosblade-1.2.0/">>~/.bashrc')
                r.exec_cmd('source ~/.bashrc')
                r.upload_file("chaosblade.tar.gz", "/opt")

            select_type = select("选择被检查类型:",["主机","docker"])

            if select_type=="主机":
                type_data["type"]="host"
                type_data["dockerName"]=""
            elif select_type=="docker":
                type_data["type"]="docker"
                type_data["dockerName"]=input("请输入docker名称", name="dockerName")
            # else:
            #     popup(title="注意",content="类型选择有误！！！")
            #     select_type=None

            chaos_data = input_group("请输入chaos命令",[
                
            input("注入命令",name="execCommand"),
            input("注入命令期望返回", name="execCommandExpect",value="success"),
            ])
            
            select_res = select("选择检查方式:",["主机命令检查","http请求检查"])

            if select_res=="主机命令检查":
                check_data = input_group("host命令检查",[
                input("检查命令", name="checkCommand"),
                input("检查命令期望返回", name="checkExpect"),
                ])
                http_check_data["httpCheckCommand"]=""
            elif select_res=="http请求检查":
                
                check_data = input_group("http检查",[
                input("http检查接口地址", name="httpCheckCommand"),
                input("http method", name="httpCheckCommand"),
                input("http header", name="httpCheckCommand"),
                input("http param", name="httpCheckCommand"),
                input("http cookie", name="httpCheckCommand"),
                input("http json", name="httpCheckCommand"),
                input("http检查期望返回", name="httpExpect"),
                ])
                http_check_data["httpCheckCommand"]=check_data
            else:
                popup(title="注意",content="检查方式选择：有误！！！")

            
            temp_data.update(server_data)
            temp_data.update(chaos_data)
            temp_data.update(check_data)
            temp_data.update(type_data)
            temp_data.update(http_check_data)
            temp_data["flag"]="true"
            input_data["data"].append(temp_data)



            popup(title="可直接忽略",content=json.dumps(input_data)) 

            go = select("是否继续输入下一个测试项:",["是","否"])
            if go=="是":
                go_on=True
            elif go=="否":
                go_on=False
                runAndGetReport(input_data)
                
    except Exception as e:
        popup(title="error",content=put_text(e))
        clear('content')

            

def runAndGetReport(input_data):
    clear('content')
    '''
    generate blade test report
    :param input_data:
    :return:
    '''
    session.set_env(title='testToolKit')
    popup(title="注意",content="正在运行测试。。。") 
    url=generateHtmlReport(running(jf=input_data))
    print(url)
    close_popup()
    # put_link(app='Result',new_window=True,name='Result')
    # put_html(html=open(url,encoding='utf-8').readlines())
    put_file(content=open(url,mode="rb").read(),name="result.html",label="点击下载简报")



import random,time
import os,sys
import platform as pf
curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(rootPath)
from time import sleep
import shutil

from loguru import logger
from pywebio.input import input, FLOAT,NUMBER,input_group,select, textarea,file_upload,checkbox,radio,actions
from pywebio.output import close_popup, popup,output, put_file, put_html, put_image, put_markdown, put_text,popup,put_link,put_code,put_row,put_processbar,set_processbar,put_error,put_warning,toast,put_grid,put_button,put_table,use_scope,span,clear,remove,get_scope
from pywebio import start_server,session,platform
from core.bladeTest.main import RemoteRunner,generateHtmlReport,running
from core.jmeterTool.swagger2jmeter import swagger2jmeter
from core.jmeterTool.har2jmeter import har2jmeter
from core.xmind2excel import makeCase
from core.utils import CFacker,getDateTime,parseJmeterXml
from core.mqttUtil import NormalMqttGetter,NormalMqttSender
from core.kafkaUtil import general_sender,continue_orderMsg,general_orderMsg,general_orderMsgWithFilter,kafkaFetchServerWithFilter,kafkaFetchServer
from functools import partial
from multiprocessing import Process
import decimal,websockets,asyncio
import json
from functools import partial
import getpass



@use_scope('content',clear=True)
def jmeterRule():
    session.set_env(title='testToolKit')
    
    put_markdown(r'''
    # 使用须知：
    ## 1.从本地址下载jmeter.zip包，已经下载并支持http,jdbc,kafka,mqtt等消息的发送插件，无需再次下载
    ## 2.支持直接拷贝开发出获得的代码进行运行，如缺少maven依赖包，可以用groovy进行包依赖管理，本下载包已经加入了包管理
    ```
    引入包参考：
    @Grapes([
        @Grab(group='org.apache.kafka', module='kafka-clients', version='2.5.1'),
        @Grab(group='org.apache.kafka', module='kafka_2.13', version='2.5.1'),
        @Grab(group='com.fasterxml.jackson.core', module='jackson-core', version='2.12.3')
        ])
    ```
    ## 3.如需要使用平台运行脚本，并需要运行时修改特定的参数项，可以按照如下格式修改：
    ```
    参数名称：authDbName  值：${__P(authDbName,auth_center)} 备注：xxx
    ```
    ## 4.如果需要查看结果，需要将请求的sampler命名格式 以名称+路径的方式存储，如：
    登录/login/index

    ## 5.jemter脚本中，需要注意
    
    - user defined variables 组件中不要带有jmeter的自定义变量如：${__time(,)}
    - 需要有自定义变量的时候，可以采用在jsr233 sampler中采用vars.put(name,'value')的方式进行定义，如 vars.put('mytime',${__time(,)})

    ''', lstrip=True).show()

@use_scope('content',clear=True)
def jmeterDownload():
    session.set_env(title='testToolKit')
    filename='apache-jmeter-5.4.1.zip'
    libpath=sys.path
    print(f'libpath: {libpath}')
    for one in libpath:
        if one.endswith("site-packages"):
            location1Prefx=one
            print(f"location1Prefx: {location1Prefx}")
    location1=location1Prefx+os.sep+"core"+os.sep+"jmeterTool"+os.sep+"jmeterzip"+os.sep
    print(f"location1: {location1}")
    if os.path.exists(location1+filename):
        put_file(content=open(location1+filename,mode="rb").read(),name=filename,label="点击下载软件")
    else:
        jmeterzip = file_upload("如果你是管理员，上传你的jmeter压缩包，注意jmeter压缩包解压后直接可以看到bin目录，多层级可能导致运行问题",accept="*.jmx",placeholder='选择jmeter zip文件')
        scriptName="apache-jmeter-5.4.1.zip"#jmeterzip['filename']
        # user = getpass.getuser()
        # passwd = getpass.getpass("请输入您的密码:")
        open(location1+scriptName, 'wb').write(jmeterzip['content'])


@use_scope('content',clear=True)
def jmeterRun():
    '''this is using jmeter.bat or jmeter.sh to run the script you have generated and share with other'''
    session.set_env(title='testToolKit')
    clear('content')
    userPath=os.path.expanduser('~')
    resultFile=userPath+os.sep+"out.jtl"
    if os.path.exists(resultFile):
        os.remove(resultFile)

    reportDir=userPath+os.sep+"report"

    if not os.path.exists(reportDir):
        os.mkdir(reportDir)
        tmp = os.popen('chmod -R 755 '+reportDir+os.sep).readlines()
    else:
        shutil.rmtree(reportDir)
        os.mkdir(reportDir)
        tmp = os.popen('chmod -R 755 '+reportDir+os.sep).readlines()

    script_f = file_upload("上传jmx脚本文件",accept="*.jmx",placeholder='选择jmx文件')
    scriptName=script_f['filename']

    open(userPath+os.sep+scriptName, 'wb').write(script_f['content'])

    

    flag = actions('确认', ['是', '否'],
                        help_text='是否需要参数文件?')
    if flag=="是":
        while flag=="是":
            data_f = file_upload("上传脚本文件所需的数据文件",accept="*.csv",placeholder='选择csv文件')
            open(userPath+os.sep+data_f['filename'], 'wb').write(data_f['content'])
            # 简单的操作
            flag = actions('确认', ['是', '否'],
                            help_text='是否需要继续上传参数文件?')

    filename='apache-jmeter-5.4.1.zip'
    plat=pf.platform().split('-')[0]
    # location1=os.path.join(sys.exec_prefix, 'jmeterTool')+os.sep+"jmeterzip"+os.sep
    libpath=sys.path
    print(f'libpath: {libpath}')
    for one in libpath:
        if one.endswith("site-packages"):
            location1Prefx=one
            # print(location1Prefx)

    # if "core" in location1Prefx:
    #     location1=location1Prefx+os.sep+"jmeterTool"+os.sep+"jmeterzip"+os.sep
    # else:
    #     location1=location1Prefx+os.sep+"core"+os.sep+"jmeterTool"+os.sep+"jmeterzip"+os.sep
    location1=location1Prefx+os.sep+"core"+os.sep+"jmeterTool"+os.sep+"jmeterzip"+os.sep
    location=os.path.abspath(os.getcwd())+os.sep
    # print(plat)
    # print(location)
    print(f"location1: {location1}")

    if "wind" in plat.lower():
        import time
        put_processbar(label='准备中',name='bar',init=0.15,auto_close=True)
        shutil.unpack_archive(location1+filename, userPath)
        for i in range(2, 11):
            set_processbar('bar', i / 10)
            time.sleep(0.1)
    else:
        import time
        put_processbar(label='准备中',name='bar',init=0.15,auto_close=True)
        shutil.unpack_archive(location1+filename, userPath)
        for i in range(2, 11):
            set_processbar('bar', i / 10)
            time.sleep(0.1)
        tmp = os.popen('chmod -R 777 '+userPath+os.sep+"apache-jmeter-5.4.1"+os.sep).readlines()

    jmxFilePath=userPath+os.sep+scriptName
    # print(f'path: {jmxFilePath}')
    jmxParamList=parseJmeterXml(jmxFilePath)

    # print(type(jmxParamList))
    print(jmxParamList)


    if jmxParamList==False or jmxParamList==None:
        toast('jmx脚本没有参数,即将直接运行')
        runComd=userPath+os.sep+"apache-jmeter-5.4.1"+os.sep+"bin"+os.sep+"jmeter -n -t "+jmxFilePath+" "+"-l "+reportDir+os.sep+scriptName+getDateTime(format='%Y%m%d%H%M%S')+".jtl"+" -e -o "+reportDir
        # reportComd=userPath+os.sep+"apache-jmeter-5.4.1"+os.sep+"bin"+os.sep+"jmeter -g out.jtl -o "+reportDir
        print(runComd)
        # tmp = os.popen(runComd).readlines()
        
        put_processbar(label='运行中',name='bar',init=0.15,auto_close=True)
        tmp = os.popen(runComd).readlines()
        # os.popen(reportComd)
        for i in range(2, 11):
            set_processbar('bar', i / 10)
            time.sleep(0.1)
        put_text("\n".join(tmp))
    else:
        paraList=[]
        for one in jmxParamList:
            if len(one)==3:
                paraList.append(input(label=one[0]+": "+one[2], name=one[0]+str(CFacker().get_it("random_number",8)),value=one[1]))
            elif len(one)<=2:
                paraList.append(input(label=one[0]+": "+"No Desc", name=one[0]+str(CFacker().get_it("random_number",8)),value=one[1]))

        info = input_group(scriptName+": 脚本参数",paraList)

        paramStr=""
        for k, v in info.items():
            paramStr=paramStr+("-J"+k+" "+v+" ")
        # print(paramStr)

        runComd=userPath+os.sep+"apache-jmeter-5.4.1"+os.sep+"bin"+os.sep+"jmeter -n -t "+jmxFilePath+" "+paramStr+"-l "+reportDir+os.sep+scriptName+getDateTime(format='%Y%m%d%H%M%S')+".jtl"+" -e -o "+reportDir
        print(runComd)
        # reportComd=userPath+os.sep+"apache-jmeter-5.4.1"+os.sep+"bin"+os.sep+"jmeter -g out.jtl -o "+reportDir
    

        put_processbar(label='运行中',name='bar',init=0.15,auto_close=True)
        tmp = os.popen(runComd).readlines()
        # os.popen(reportComd)
        for i in range(2, 11):
            set_processbar('bar', i / 10)
            time.sleep(0.1)
        put_text("\n".join(tmp))

    # shutil.make_archive("report", "zip",reportDir)
    put_link('查看报告',url='/static/index.html',new_window=True)


@use_scope('content',clear=True)
def jmeterDeploy():
    session.set_env(title='testToolKit')
    filename='apache-jmeter-5.4.1.zip'
    libpath=sys.path
    print(f'libpath: {libpath}')
    for one in libpath:
        if one.endswith("site-packages"):
            location1Prefx=one
            print(f"location1Prefx: {location1Prefx}")
    location1=location1Prefx+os.sep+"core"+os.sep+"jmeterTool"+os.sep+"jmeterzip"+os.sep
    print(f"location1: {location1}")
    if not os.path.exists(location1+filename):
        # put_file(content=open(location1+filename,mode="rb").read(),name=filename,label="点击下载软件")
    # else:
        jmeterzip = file_upload("如果你是管理员，上传你的jmeter压缩包，注意jmeter压缩包解压后直接可以看到bin目录，多层级可能导致运行问题",accept="*.jmx",placeholder='选择jmeter zip文件')
        scriptName="apache-jmeter-5.4.1.zip"#jmeterzip['filename']
        # user = getpass.getuser()
        # passwd = getpass.getpass("请输入您的密码:")
        open(location1+scriptName, 'wb').write(jmeterzip['content'])
    


    flag = actions('确认', ['是', '否'], help_text='是否需要参数文件?')
    
    if flag=="是":
        while flag=="是":
            data_f = file_upload("上传脚本文件所需的数据文件",accept="*.csv",placeholder='选择csv文件')
            open(userPath+os.sep+data_f['filename'], 'wb').write(data_f['content'])
            # 简单的操作
            flag = actions('确认', ['是', '否'],
                            help_text='是否需要继续上传参数文件?')




@use_scope('content',clear=True)
def jmeterScriptGen():
    '''
    this just invoke the lib for generate jmeter script
    :return:
    '''
    session.set_env(title='testToolKit')
    clear('content')
    select_type = select("选择你要做的操作:",["自动化规范","标准包下载","swagger转脚本","har转脚本","自测工作台","分布式部署","docker打包部署(未开始)","标准包添加插件(未开始)"])
    if select_type=="自动化规范":
        jmeterRule()
    elif select_type=="标准包下载":
        jmeterDownload()
    elif select_type=="swagger转脚本":
        url=input('输入swagger地址：example:http://192.168.xxx.xxx:port/space_name/v2/api-docs')
        # print(url)
        location=os.path.join(sys.exec_prefix, 'jmx')+os.sep
        # print(location)
        swagger2jmeter(url,location)
        file_location=None
        # location=os.path.abspath(os.path.dirname(__file__))+os.sep+'jmx'+os.sep
        # print(location)
        for x,y,z in os.walk(location):
            file_location=location+"".join(z)
            break
        # print(file_location)
        put_file(content=open(file_location,mode="rb").read(),name=file_location.split(os.sep)[-1],label="点击下载jmeter脚本")
        os.remove(file_location)
    elif select_type=="har转脚本":
        f = file_upload("上传har文件，可以从fidder, charlse, chrome开发者工具中导出",accept="*.har",placeholder='选择har文件')
        open('temp.har', 'wb').write(f['content'])
        har2jmeter('temp.har')
        location=os.path.abspath('.')+os.path.sep+"autoGen.jmx"
        # print(location)
        put_file(content=open(location,mode="rb").read(),name="autoGen.jmx",label="点击下载jmeter脚本")
    elif select_type=="自测工作台":
        jmeterRun()
    elif select_type=="分布式部署":
        jmeterDeploy()


import sys,os
sys.path.extend(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
from pywebio.platform.flask import webio_view
from flask import Flask,request,render_template,session
from flask import redirect
from werkzeug.routing import BaseConverter
# from core.bladeTest.interactive import myapp


class RegexConverter(BaseConverter):
    def __init__(self, url_map, *items):
        super(RegexConverter, self).__init__(url_map)
        self.regex=items[0]
    


staticPath=os.path.abspath(os.path.expanduser('~'))
app = Flask(__name__)#,static_url_path="/static",static_folder="static",template_folder="templates")
app.url_map.converters['reg']=RegexConverter
g_res=None

# `task_func` is PyWebIO task function
# app.add_url_rule('/testTool', 'webio_view', webio_view(myapp),methods=['GET', 'POST', 'OPTIONS'])  # need GET,POST and OPTIONS methods

@app.route('/<reg(".*"):mypath>')
def testTool(mypath):
    global g_res
    return g_res


# @app.route('/kafka')
# def kafkaClient():
#     portNum=request.args.get('portNum')
#     ip=request.args.get('ip')
#     print(session.get('host'))
#     # return redirect(f'/static/kafkaWebClient{portNum}.html')
#     return render_template('kafkaWebClient.html',ip=ip,portNum=portNum)

def run(port=8899,retStr="Not set when your start app"):
    global g_res
    g_res=retStr
    app.run(host='0.0.0.0', port=port)


if __name__=='__main__':
    app.run(host='0.0.0.0', port=8899)


import sys
import os,time
# print("###"+os.path.abspath(os.path.dirname(os.path.dirname(os.getcwd()))))
sys.path.append(os.path.abspath(os.path.dirname(os.path.dirname(os.getcwd())))) 

from loguru import logger
from pywebio.input import input, FLOAT,NUMBER,input_group,select, textarea,file_upload,checkbox
from pywebio.output import close_popup, output, put_file, put_html, put_image, put_markdown, put_text,popup,put_link,put_code,put_row
from pywebio import session
from pywebio import start_server
from core.bladeTest.main import RemoteRunner,generateHtmlReport,running
from core.kafkaUtil import kafkaOper,general_sender,multi_topic_listener,general_listener
import json
from core.xmind2excel import makeCase
from core.utils import swagger2jmeter,CFacker
import decimal,datetime
from random import randint



# def kafkaUi():
#     session.set_env(title='testToolKit')
#     check_data = input_group("kafka check tool",[
#             input("kafka topics", name="topic",help_text='which topic you want to send or listen'),
#             input("kafka server:port", name="address",help_text='format like 0.0.0.0:9092'),
#             input("credentials", name="credentials",help_text='password of the kafka'),
#             input("filter flag", name="flag",help_text='only allowed "json" or "regx"'),
#             input("filter pattern", name="pattern",help_text='how to find the key by this pattern'),
#             input("assert key", name="key",help_text='what key you want to find'),
#             input("messages",name="msg",help_text='fill only when you need send messages'),
#             ])
#     if check_data['msg']!="" or check_data['msg']!=None:
#         general_sender(check_data['topic'],check_data['address'],check_data['msg'])
#     else:
#         general_listener(check_data['topic'],check_data['address'],check_data['flag'],check_data['pattern'],check_data['key'])

def continueSend():
    while True:
        server="127.0.0.1:9092"
        time.sleep(5)
        num=datetime.datetime.now()
        msg={"data":{"date":"2021-01-15"}}

        general_sender('test2',server,json.dumps(msg))
        print(f"{msg} sent")
        # general_sender('vv', server, 'bbb'+str(num))
        # general_sender('dd', server, 'ccc'+str(num))

if __name__=="__main__":
    # kafkaUi()
    continueSend()